# analysis script for Experiment 1 Nijveld, ten Bosch & Ernestus (under review) #
# ran under R version 3.4.1
#### contains:  description of participants
####            description of materials (frequency of occurrence, word duration)
####            analysis of **accuracy**
####                -description of average error rates
####                -priming of primes by female speaker
####                -final model 
####                -competing = alternative model (incl. discussion of slopes)
####                -cognate analysis
####                -proficiency analysis
####            analysis of **RT**
####                -description of average RTs
####                -priming of primes by female speaker
####                -final model (incl. discussion of slopes)
####                -competing model
####                -cognate analysis
####                -proficiency analysis
####                -code to generate barplots RT + helper functions
##################################################

# preliminaries: load libraries, load data
# load libraries
library(plyr)
library(lme4)
library(ggplot2)

d=read.table('ds_EXP1_voice.txt',T)
colnames(d) # overview
# [1] "subject"           "exp_name"          "trial"             "sub_experiment"    "lexdec_part"      
# [6] "handedness"        "correctresponse"   "condition"         "withinPartTrial"   "word"             
# [11] "word.nonword"      "response"          "RT"                "correctness"       "wavduration"      
# [16] "RTprime"           "RTprev"            "lag"               "correctPrime"      "score_lextale"    
# [21] "gender"            "age"               "yrsListenedEN"     "levelENlistening"  "nativeLanguage"   
# [26] "stimtype"          "prime.target"      "speaker"           "stimulus"          "count_per_million"
# [31] "X2log"    

################### PARTICIPANTS
d1=d[d$subject!="M_pp14"&d$subject!="N_pp4",] # 2 participants exluded a-priori due to technical issues

### participant characteristics per participant group
# Dutch (Nijmegen)
dN = d1[d1$nativeLanguage=="dutch",] # nrow 40
ppN = unique(dN[, c("subject", "handedness", "gender", "age", "score_lextale", "levelENlistening","nativeLanguage")]) 
summary(ppN)
# subject   handedness    gender        age        score_lextale   levelENlistening nativeLanguage
# N_pp1  : 1   left : 6   female:32   Min.   :18.00   Min.   :51.25   Min.   :2.00     dutch  :40    
# N_pp10 : 1   right:34   male  : 8   1st Qu.:19.00   1st Qu.:65.94   1st Qu.:4.75     english: 0    
# N_pp11 : 1                          Median :20.00   Median :71.25   Median :5.00     spanish: 0    
# N_pp12 : 1                          Mean   :20.98   Mean   :73.59   Mean   :4.75                   
# N_pp13 : 1                          3rd Qu.:22.00   3rd Qu.:81.56   3rd Qu.:5.00                   
# N_pp14 : 1                          Max.   :27.00   Max.   :96.25   Max.   :6.00                   
# (Other):34                                                          NA's   :4                                                                 
sd(ppN$score_lextale)
# [1] 10.66006
sd(ppN$levelENlistening) # to do

# Spanish (Madrid)
dM = d1[d1$nativeLanguage=="spanish",] # nrow 33
ppM = unique(dM[, c("subject", "handedness", "gender", "age", "score_lextale", "levelENlistening","nativeLanguage")]) 
# subject   handedness    gender        age        score_lextale   levelENlistening nativeLanguage
# M_pp1  : 1   left : 5   female:12   Min.   :19.00   Min.   :50.00   Min.   :2.000    dutch  : 0    
# M_pp10 : 1   right:28   male  :21   1st Qu.:20.00   1st Qu.:60.00   1st Qu.:4.000    english: 0    
# M_pp11 : 1                          Median :22.00   Median :68.75   Median :4.000    spanish:33    
# M_pp12 : 1                          Mean   :21.79   Mean   :66.93   Mean   :4.273                  
# M_pp13 : 1                          3rd Qu.:23.00   3rd Qu.:73.75   3rd Qu.:5.000                  
# M_pp15 : 1                          Max.   :31.00   Max.   :83.75   Max.   :6.000                  
# (Other):27   
sd(ppM$score_lextale)
# [1] 8.872419
sd(ppM$levelENlistening)
# [1] 0.9107939

dC = d1[d1$nativeLanguage=="english",] # nrow 40
ppC = unique(dC[, c("subject", "handedness", "gender", "age")]) 
# subject   handedness    gender        age       
# C_pp1  : 1   left : 6   female:29   Min.   :18.00  
# C_pp10 : 1   right:34   male  :11   1st Qu.:19.00  
# C_pp11 : 1                          Median :20.00  
# C_pp12 : 1                          Mean   :21.27  
# C_pp13 : 1                          3rd Qu.:23.00  
# C_pp14 : 1                          Max.   :34.00  
# (Other):34                                         

### LEXTALE proficiency scores comparison of the non-native listeners
nn=rbind(ppN, ppM)
lm=lm(score_lextale~nativeLanguage,data=nn)
summary(lm)
# Call:
#   lm(formula = score_lextale ~ nativeLanguage, data = nn)
# 
# Residuals:
#   Min       1Q   Median       3Q      Max 
# -22.3438  -7.3437   0.1563   6.8182  22.6562 
# 
# Coeficients:
#                         Estimate Std. Error t value Pr(>|t|)    
#   (Intercept)             73.594      1.564  47.041  < 2e-16 ***
#   nativeLanguagespanish   -6.662      2.327  -2.863  0.00551 ** (in-text)
#   ---
#   Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
# 
# Residual standard error: 9.894 on 71 degrees of freedom
# Multiple R-squared:  0.1035,	Adjusted R-squared:  0.09088 
# F-statistic: 8.197 on 1 and 71 DF,  p-value: 0.005511

######################## MATERIALS

# frequency of occurrence (as obtained from the British National Corpus)
words=d1[d1$stimtype=="ld_experimental_real"|d1$stimtype=="ld_distractorfoil_real",]
words=droplevels(words)
w = unique(words[, c("word", "X2log", "stimtype")]) 

tapply(w$X2log, w$stimtype, mean)
# ld_distractorfoil_real   ld_experimental_real 
# 4.212929               4.480420 

tapply(w$X2log, w$stimtype, range)
# $ld_distractorfoil_real
# [1] 0.831877 7.983564
# 
# $ld_experimental_real
# [1] 0.189034 8.079058

tapply(w$X2log, w$stimtype, sd)
# ld_distractorfoil_real   ld_experimental_real 
# 2.581387                1.943066 

dep.t.test = t.test(w$X2log ~ w$stimtype, paired=FALSE)
# Welch Two Sample t-test
# 
# data:  w$X2log by w$stimtype
# t = -0.29998, df = 12.667, p-value = 0.7691
# alternative hypothesis: true difference in means is not equal to 0
# 95 percent confidence interval:
#   -2.199018  1.664036
# sample estimates:
#   mean in group ld_distractorfoil_real   mean in group ld_experimental_real 
# 4.212929                             4.480420 

# word duration of two speakers
d2=d1[d1$sub_experiment=="lexdecvoice"&d1$stimtype=="ld_experimental_real",]
tokens = unique(d2[, c("stimulus", "wavduration", "speaker", "prime.target")]) 
tokens$wavduration=as.numeric(as.character(tokens$wavduration))
round(tapply(tokens$wavduration, tokens$speaker, mean))
# sF  sM 
# 642 561 

round(tapply(tokens$wavduration, tokens$speaker, sd))
# sF sM 
# 62 72 

tapply(tokens$wavduration, tokens$speaker, range)
# $sF
# [1] 551 831
# 
# $sM
# [1] 410 740

# for question reviewer 2: what is the difference in duration between primes and targets of the male speaker?
tokensM=tokens[tokens$speaker=="sM",]

tapply(tokensM$wavduration, tokensM$prime.target, mean)
# prime target 
# 551.2  571.7 

# range
tapply(tokensM$wavduration, tokensM$prime.target, range)
#$prime
# 450 696 (was 410)
#
#$target
# 481 740

tapply(tokensM$wavduration, tokensM$prime.target, sd)
# prime   target 
# 73.70135 70.30287 

dep.t.test = t.test(tokensM$wavduration ~ tokensM$prime.target, paired=T)
# Paired t-test
# 
# data:  tokensM$wavduration by tokensM$prime.target
# t = -1.0832, df = 29, p-value = 0.2877
# alternative hypothesis: true difference in means is not equal to 0
# 95 percent confidence interval:
#   -59.20846  18.20846
# sample estimates:
#   mean of the differences 
# -20.5 

################### RESULTS AND DISCUSSION

#### participant and word exclusion if exceeding 2.5 sd from grand mean
# participants
d1=d[d$sub_experiment=="lexdecvoice"&d$stimtype=="ld_experimental_real"&d$prime.target=="target",]
d2=d1[d1$subject!="M_pp14"&d1$subject!="N_pp4",] # technical issues
d2=droplevels(d2)

correct = table(d2$subject,d2$correctness)
new = data.frame(correct)
subsetnew = new[new$Var2 == "incorrect",]
m = mean(subsetnew$Freq)
sd = sd(subsetnew$Freq)
exclude = subsetnew[subsetnew$Freq > m+(2.5*sd),]
# Var1      Var2 Freq
# 164 M_pp17 incorrect   14
# 176 M_pp28 incorrect    9
# 195 N_pp14 incorrect    9

# words
correct = table(d2$word,d2$correctness)
new = data.frame(correct)
subsetnew = new[new$Var2 == "incorrect",]
m = mean(subsetnew$Freq) 
sd = sd(subsetnew$Freq) 

exclude = subsetnew[subsetnew$Freq > m+(2.5*sd),]
# Var1      Var2 Freq
# 57 saloon incorrect   38

##### recompile data without these pp & word
d3 = d2[d2$word!="saloon",]
d4=d3[d3$subject!="M_pp17"&d3$subject!="M_pp28"&d3$subject!="N_pp14",] 


#### ACCURACY ANALYSIS ######

### exclude targets with incorrect primes (exclusion percentage reported below)
d5=d4[d4$correctPrime=="correct",]

round(100*(nrow(d4)-nrow(d5))/nrow(d4)) # 8% 

# examine 8% of removed data (242 rows)
setdiffDF <- function(A, B){
  f <- function(A, B)
    A[!duplicated(rbind(B, A))[nrow(B) + 1:nrow(A)], ]
  df1 <- f(A, B)
  df2 <- f(B, A)
  rbind(df1, df2)
}
a=setdiffDF(d4, d5)
summary(a$nativeLanguage)
# dutch english spanish 
# 86      34     122 


#### average error rates after this step
dxtabs = xtabs(~correctness, data=d5) # across groups
round(100 * prop.table(dxtabs))
# correct incorrect 
# 97         3 

dxtabs2 = xtabs(~correctness+nativeLanguage, data=d5) # per native language group
round(100 * prop.table(dxtabs2,2))
# nativeLanguage
# nativeLanguage
# correctness dutch english spanish
# correct      97      98      96
# incorrect     3       2       4

# calculate SDs from mean
correct_subj = data.frame(table(d5$subject,d5$correctness)) # across groups
correct_subj2 = correct_subj[correct_subj$Var2 == "incorrect",]
sd = sd(correct_subj2$Freq) 
# divide by 29 experimental words
round(sd/29*100) # 3

# listener group subsets
# English
dE=d5[d5$nativeLanguage=="english",]
dE=droplevels(dE)
correct_subj = data.frame(table(dE$subject,dE$correctness))
correct_subj2 = correct_subj[correct_subj$Var2 == "incorrect",]
sd = sd(correct_subj2$Freq) 
# divide by 29 experimental words
round(sd/29*100) # 4

# Dutch
dD=d5[d5$nativeLanguage=="dutch",]
dD=droplevels(dD)
correct_subj = data.frame(table(dD$subject,dD$correctness))
correct_subj2 = correct_subj[correct_subj$Var2 == "incorrect",]
sd = sd(correct_subj2$Freq) 
# divide by 29 experimental words
round(sd/29*100) # 3

# Spanish
dS=d5[d5$nativeLanguage=="spanish",]
dS=droplevels(dS)
correct_subj = data.frame(table(dS$subject,dS$correctness))
correct_subj2 = correct_subj[correct_subj$Var2 == "incorrect",]
sd = sd(correct_subj2$Freq) 
# divide by 29 experimental words
round(sd/29*100) # 3


###################### intermediate analysis #########################

#### review round 3: rev 3 asks about whether the task in part 2 is something different from lexdec. This may imply that the RTs on the real words in part 1 differ significantly from the RTs on the non-repeated words in part 2


d1 = d[d$lexdec_part=="ld_part1"|d$lexdec_part=="ld_part2",] # consider targets and primes
d2=d1[d1$subject!="M_pp14"&d1$subject!="N_pp4",] # technical issues
d2=droplevels(d2)
d3 = d2[d2$word!="saloon",] # too many errors
d4=d3[d3$subject!="M_pp17"&d3$subject!="M_pp28"&d3$subject!="N_pp14",] # too many errors
dtmp = d4[d4$subject == "N_pp1" & d4$lexdec_part=="ld_part2",]
# non-repeated targets have lag=="-" or condition==NA

dd10 = d4[(d4$lexdec_part=="ld_part1") | (d4$lexdec_part=="ld_part2" & d4$lag=="-"), ]
dd11 = dd10[dd10$correctness=="correct",] # 12936 out of 14075 kept
dd12 = dd11[!is.na(dd11$lexdec_part),] # 8211 kept, 6381 part1, 1830 part2

dd12$RTprevN = as.numeric(as.character(dd12$RTprev))
dd13 = dd12[(300 < dd12$RT) & (300 < dd12$RTprevN) & (dd12$RTprevN < 1900),] # 7636 kept
dd13$logRT = log(dd13$RT)
dd13$logRTprev = log(dd13$RTprevN)
dd13$logwavduration = log(as.numeric(as.character(dd13$wavduration)))


m1 = lmer(logRT ~ lexdec_part + logRTprev + logwavduration + (1|word) + (1|subject), data=dd13, REML=F)


Random effects:
 Groups   Name        Variance Std.Dev.
 subject  (Intercept) 0.019956 0.14127 
 word     (Intercept) 0.009893 0.09947 
 Residual             0.036108 0.19002 
Number of obs: 7585, groups:  subject, 110; word, 85

Fixed effects:
                    Estimate Std. Error t value
(Intercept)         5.171576   0.200476  25.796
lexdec_partld_part2 0.005209   0.026113   0.199 <<< this is indeed what we expected to see
logRTprev           0.129467   0.012168  10.640
logwavduration      0.150133   0.028186   5.327
########################## end of intermediate analysis ########################



#### first analysis: priming of primes by female speaker on accuracy

d1 = d[d$lexdec_part=="ld_part1"|d$lexdec_part=="ld_part2",] # consider targets and primes
d2=d1[d1$subject!="M_pp14"&d1$subject!="N_pp4",] # technical issues
d2=droplevels(d2)
d3 = d2[d2$word!="saloon",] # too many errors
d4=d3[d3$subject!="M_pp17"&d3$subject!="M_pp28"&d3$subject!="N_pp14",] # too many errors
p=d3[d3$prime.target=="prime"&d3$speaker=="sF",]# correct primes of female speaker only
ctmm=d3[d3$prime.target=="target"&d3$correctPrime=="correct"&d3$condition=="speaker_mismatch",] # targets with correct primes amd spoken by female speaker only
d5=rbind(p,ctmm)
d5$mycorr = (d5$correctness == "correct")+0  # for interpretation: 0 is lowest likehood correct, 1 maximal likelihood correct

d5.glmer1  = glmer(mycorr ~ prime.target   + (1|subject) + (1|word), data = d5, family = "binomial")
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)          3.1658     0.2319  13.649  < 2e-16 ***
#   prime.targettarget   0.5801     0.1700   3.412 0.000645 ***
#   ---
#   Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

#### final accuracy model
# redo data prep
d=read.table('ds_EXP1_voice.txt',T)
d1 = d[d$lexdec_part=="ld_part2"&d$stimtype=="ld_experimental_real",]
d2=d1[d1$subject!="M_pp14"&d1$subject!="N_pp4",] # technical issues
d3 = d2[d2$word!="saloon",] # too many mistakes
d4=d3[d3$subject!="M_pp17"&d3$subject!="M_pp28"&d3$subject!="N_pp14",] # too many mistakes
d5=d4[d4$correctPrime=="correct",]
d5=droplevels(d5)

d5$nativeLanguage=relevel(d5$nativeLanguage, "english")
d5$groupcontrast=d5$nativeLanguage
#d3 = droplevels(d3)

contrasts(d5$groupcontrast) = rbind(c(2/3, 0), c(-1/3, 1/2), c(-1/3, -1/2)) # helmert coding
contrasts(d5$groupcontrast)
# [,1] [,2]
# english  0.6666667  0.0
# dutch   -0.3333333  0.5
# spanish -0.3333333 -0.5
d5$mycorr = (d5$correctness == "correct")+0 
d5$wavduration = as.numeric(as.character(d5$wavduration))
d5$logwavduration = log(d5$wavduration)
d5$condition=relevel(d5$condition, "speaker_mismatch") # value on intercept
d5$condition_rel=relevel(d5$condition, 'speaker_mismatch') # relevel

#### accuracy model w/ predictors of interest (all non-significant; reported in-text)
d5.glmer3  = glmer(mycorr ~ groupcontrast * condition  + (1|subject) + (1|word), data = d5, family = "binomial") # (2nd option A2-1)
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                             3.6318     0.2324  15.629   <2e-16 ***
#   groupcontrast1                        0.3189     0.3294   0.968   0.3329   (in-text) 
# groupcontrast2                          0.2673     0.3825   0.699   0.4847    in-text
# conditionspeaker_match                  0.3757     0.2228   1.686   0.0918 .  in-text
# groupcontrast1:conditionspeaker_match   0.2668     0.4845   0.551   0.5818    
# groupcontrast2:conditionspeaker_match   0.1153     0.5307   0.217   0.8280    

######################### intermezzo louis
### try to enrich this with logwavduration, trial, lag


d=read.table('ds_EXP1_voice.txt',T)
d1 = d[d$lexdec_part=="ld_part2"&d$stimtype=="ld_experimental_real",]
d2=d1[d1$subject!="M_pp14"&d1$subject!="N_pp4",] # technical issues
d3 = d2[d2$word!="saloon",] # too many mistakes
d4=d3[d3$subject!="M_pp17"&d3$subject!="M_pp28"&d3$subject!="N_pp14",] # too many mistakes
d5=d4[d4$correctPrime=="correct",]
d5=droplevels(d5)

d5$nativeLanguage=relevel(d5$nativeLanguage, "english")
d5$groupcontrast=d5$nativeLanguage
#d3 = droplevels(d3)

contrasts(d5$groupcontrast) = rbind(c(2/3, 0), c(-1/3, 1/2), c(-1/3, -1/2)) # helmert coding
d5$mycorr = (d5$correctness == "correct")+0 
d5$wavduration = as.numeric(as.character(d5$wavduration))
d5$logwavduration = log(d5$wavduration)
d5$condition=relevel(d5$condition, "speaker_mismatch") # value on intercept
d5$condition_rel=relevel(d5$condition, 'speaker_mismatch') # relevel


d5.glmer3B  = glmer(mycorr ~ groupcontrast * condition + trial + (1|subject) + (1|word), data = d5, family = "binomial") # diverges
d5.glmer3B  = glmer(mycorr ~ groupcontrast * condition + logwavduration + (1|subject) + (1|word), data = d5, family = "binomial") # diverges
d5.glmer3B  = glmer(mycorr ~ groupcontrast * condition + X2log + (1|subject) + (1|word), data = d5, family = "binomial") # converges with Nelder-Mead minimizer
d5.glmer3B  = glmer(mycorr ~ groupcontrast * condition + lag + (1|subject) + (1|word), data = d5, family = "binomial") # takes a long time and diverges











#############


# same model with other level of speaker match on intercept
d5.glmer3_rel  = glmer(mycorr ~ groupcontrast * condition_rel  + (1|subject) + (1|word), data = d5, family = "binomial")
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                                 3.6318     0.2324  15.629   <2e-16 ***
#   groupcontrast1                              0.3189     0.3294   0.968   0.3329    
# groupcontrast2                              0.2673     0.3825   0.699   0.4847    
# condition_relspeaker_match                  0.3757     0.2228   1.686   0.0918 .  
# groupcontrast1:condition_relspeaker_match   0.2668     0.4845   0.551   0.5818    
# groupcontrast2:condition_relspeaker_match   0.1153     0.5307   0.217   0.8280    

### competing accuracy model with all predictors that did not lead to convergence errors
# (lag/logwavduration/trial: converge errors)
d5.glmer4  = glmer(mycorr ~ groupcontrast + condition + X2log + (1|subject) + (1|word), data = d5, family = "binomial")
# Random effects:
#   Groups  Name        Variance Std.Dev.
# subject (Intercept) 0.3782   0.615   
# word    (Intercept) 0.4651   0.682   
# Number of obs: 2948, groups:  subject, 110; word, 29
# 
# Fixed effects:
#                         Estimate Std. Error z value Pr(>|z|)    
# (Intercept)            3.6317912  0.4788837   7.584 3.35e-14 ***
# groupcontrast1         0.4264882  0.2704348   1.577    0.115    
# groupcontrast2         0.3192315  0.3055748   1.045    0.296    
# conditionspeaker_match 0.3688098  0.2215912   1.664    0.096 .  
# X2log                  0.0004064  0.0902777   0.005    0.996    
# ---
#   Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

d5.glmer4_rel  = glmer(mycorr ~ groupcontrast + condition_rel + X2log + (1|subject) + (1|word), data = d5, family = "binomial") 
# Fixed effects:
#                             Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                3.6317912  0.4788837   7.584 3.35e-14 ***
# groupcontrast1             0.4264882  0.2704348   1.577    0.115    
# groupcontrast2             0.3192315  0.3055748   1.045    0.296    
# condition_relspeaker_match 0.3688098  0.2215912   1.664    0.096 .  
# X2log                      0.0004064  0.0902777   0.005    0.996    

## louis: try to add lag, logwavduration, trial.
## RESULT: Adding lag leads to very long minimization time, and leads to divergence issues
## adding logwavduration also leads to a divergent model
## we can however add trial:

d5.glmer4_relB  = glmer(mycorr ~ groupcontrast + condition_rel + X2log + trial + logwavduration  + (1|subject) + (1|word), data = d5, family = "binomial") 

Random effects:
 Groups  Name        Variance Std.Dev.
 subject (Intercept) 0.3803   0.6167  
 word    (Intercept) 0.3219   0.5673  
Number of obs: 2948, groups:  subject, 110; word, 29

Fixed effects:
                             Estimate Std. Error z value Pr(>|z|) # poss A2-1 w/o interaction
(Intercept)                -19.876059   8.903168  -2.232   0.0256 *
groupcontrast1               0.425024   0.270600   1.571   0.1163  
groupcontrast2               0.314928   0.305893   1.030   0.3032  
condition_relspeaker_match   0.367265   0.221613   1.657   0.0975 .
X2log                        0.006193   0.084339   0.073   0.9415  
trial                        0.006987   0.005966   1.171   0.2415  
logwavduration               3.587946   1.405503   2.553   0.0107 *

# use competing model to test for **random slopes** (more conservative than on final model with no fixed effects)
## listener group by word: convergence failure
d5.glmer5  = glmer(correctness ~ groupcontrast + condition + X2log + (1|subject) + (1|word) + (0+groupcontrast|word), data = d5, family = "binomial")

## speaker match by word: perfect correlation random intercept/slope
d5.glmer6  = glmer(correctness ~  groupcontrast + condition + X2log +  (1|subject) + (1|word) + (0+condition|word), data = d5, family = "binomial") 
# Random effects:
#   Groups  Name                      Variance  Std.Dev. Corr
# subject (Intercept)               0.3778275 0.61468      
# word    (Intercept)               0.0001182 0.01087      
# word.1  conditionspeaker_mismatch 0.5054254 0.71093      
#         conditionspeaker_match    0.4039257 0.63555  1.00   **

## speaker match by pp: convergence failure
d5.glmer7  = glmer(correctness ~ groupcontrast + condition + X2log +  (1|subject) + (0+condition|subject) + (1|word), data = d5, family = "binomial") # c.e.


## additional analysis: cognates (on DU & SP data)
# Dutch data
cognateD=c("balloon","banana","career", "cassette", "cement", "committee", 
           "debate", "defect","defence","domain","guitar","machine","parade",
           "police","safari","salami","saloon","tobacco","tomato")
d5$cognateD=F
d5[is.element(d5$word, cognateD),]$cognateD=T
du=d5[d5$nativeLanguage=="dutch",]
du.glmer3  = glmer(mycorr ~  cognateD * condition  + (1|subject) + (1|word), data = du, family = "binomial")
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                          4.42066    0.63378   6.975 3.06e-12 ***
#   cognateDTRUE                      -1.03854    0.65386  -1.588    0.112    
# conditionspeaker_match              -0.04672    0.72432  -0.064    0.949    
# cognateDTRUE:conditionspeaker_match  0.55225    0.84828   0.651    0.515    

# Spanish data
cognateSP=c("career", "cassette", "cement","collapse","committee", 
            "debate","decline", "defect","defence","design","divorce","domain","guitar","machine","parade",
            "police","potato","safari","salami","saloon","surprise","tobacco","tomato")
d5$cognateSP=F
d5[is.element(d5$word, cognateSP),]$cognateSP=T
sp=d5[d5$nativeLanguage=="spanish",]
sp.glmer3  = glmer(mycorr ~  cognateSP * condition  + (1|subject) + (1|word), data = sp, family = "binomial")
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                            3.1117     0.5896   5.278 1.31e-07 ***
#   cognateSPTRUE                        0.2789     0.6797   0.410    0.682    
# conditionspeaker_match                 1.0828     0.8559   1.265    0.206    
# cognateSPTRUE:conditionspeaker_match  -1.1090     0.9557  -1.160    0.246      

## additional analysis: profiency
## are results the same on subset of non-native listeners with similar lextale scores?
## non-native groups are of dissimilar size (dutch group bigger) and dissimilar proficiency (dutch group higher).
# strip groups from extremes in lextale scores to get to groups of similar size (N=29) and similar lextale score (69%).
cl=d5[d5$nativeLanguage=="dutch"&d5$score_lextale<=81.25|d5$nativeLanguage=="spanish"&d5$score_lextale>51.25,]
tapply(cl$score_lextale,cl$nativeLanguage, mean)
# english    dutch  spanish 
# NA      69.14534 68.88049 

pp1 = unique(cl[, c("subject", "score_lextale","nativeLanguage")]) 
lm=lm(score_lextale~nativeLanguage,data=pp1)
summary(lm)
# Coefficients:
#                       Estimate      Std. Error    t value Pr(>|t|)    
#   (Intercept)            68.9224     1.3742  50.153   <2e-16 ***
#   nativeLanguagespanish  -0.3448     1.9435  -0.177     0.86    

# random sample of 29 English pp (obtained with function sample(englishdata,size=29)).
es=c("C_pp32", "C_pp26", "C_pp18", "C_pp16", "C_pp36", "C_pp8",  "C_pp6",  "C_pp29", "C_pp40", "C_pp15", "C_pp2", "C_pp1","C_pp11", "C_pp19", 
     "C_pp17","C_pp21", "C_pp35", "C_pp28", "C_pp38", "C_pp34", "C_pp22", "C_pp33", "C_pp24", "C_pp23", "C_pp5",  "C_pp25","C_pp30", "C_pp27", "C_pp20")
# select this sample from larger ds
e3 = d5[is.element(d5$subject, es),]
# combine native and non-native samples
cl2=rbind(cl,e3)
cl2$groupcontrast=cl2$nativeLanguage

contrasts(cl2$groupcontrast) = rbind(c(2/3, 0), c(-1/3, 1/2), c(-1/3, -1/2)) # helmert coding

cl2.glmer3  = glmer(mycorr ~ groupcontrast * condition  + (1|subject) + (1|word), data = cl2, family = "binomial")
# Fixed effects:
#   Estimate Std. Error z value Pr(>|z|)    
# (Intercept)                            3.56507    0.24895  14.321   <2e-16 ***
# groupcontrast1                         0.32638    0.36439   0.896    0.370    
# groupcontrast2                         0.16197    0.40676   0.398    0.690    
# conditionspeaker_match                 0.22247    0.23508   0.946    0.344    
# groupcontrast1:conditionspeaker_match  0.21337    0.52023   0.410    0.682    
# groupcontrast2:conditionspeaker_match -0.03838    0.54985  -0.070    0.944    

# -> similar results as accuracy analysis on full dataset


#### RT analysis ######

####  first analysis: priming of primes by female speaker
d0=d[d$sub_experiment=="lexdecvoice",]  # consider primes and targets
#d1=d0[d0$speaker=="sM",] # male speaker only
d1=d0
d2=d1[d1$stimtype=="ld_experimental_real",] # only exp words
d3=d2[d2$correctness=="correct",] # only correct responses on current trial
p=d3[d3$prime.target=="prime"&d3$speaker=="sF",]# primes by female speaker
ctmm=d3[d3$prime.target=="target"&d3$correctPrime=="correct"&d3$condition=="speaker_mismatch",] # targets with correct primes and preceded by female speaker (mismatch)
d3a=rbind(p,ctmm)
d3b=d3a[d3a$RT>0,] # only trials with responses
d4=d3b[d3b$subject!="N_pp4"&d3b$subject!="M_pp14"&d3b$subject!="M_pp17"&d3b$subject!="M_pp28"&d3b$subject!="N_pp14"&d3b$word!="saloon",] # technical issues + too many errors

d4=d3b[d3b$RTprev!="-",] 
d4$RTprev = as.numeric(as.character(d4$RTprev))
d4=d4[d4$RTprev>0,]
d4$logRTprev = log(d4$RTprev)
d4$logRT = log(d4$RT)
d4=droplevels(d4)
d5=d4[d4$logRT<mean(d4$logRT)+2.5*sd(d4$logRT)&d4$logRT>mean(d4$logRT)-2.5*sd(d4$logRT),] 

tapply(d5$RT, d5$prime.target, mean)
# prime    target 
# 1026.6377  948.5479 

# use final model RTs (see below) apart from predictor RTprime (is not sensible when in part analyzing primes)
d5.lmer3 =  lmer(logRT~ prime.target  + logRTprev +    (1|word) + (1|subject), data=d5, REML=F)
d5.lmer3a = lmer(logRT~ prime.target  + logRTprev +    (1|word) + (1|subject), data=d5[abs(scale(resid(d5.lmer3))) < 2.5,], REML=F)
summary(d5.lmer3a)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)         6.515046   0.074174  87.835
# prime.targettarget -0.080530   0.005237 -15.376
# logRTprev           0.057433   0.010285   5.584

#### main RT analysis

#### exclusion of targets with incorrect primes + incorrect targets
d0=d[d$sub_experiment=="lexdecvoice",]
d1=d0[d0$lexdec_part=="ld_part2",] # only test block (not lextale, not prime block)
d2=d1[d1$stimtype=="ld_experimental_real",] # only exp words
d2a=d2[d2$subject!="N_pp4"&d2$subject!="M_pp14"&d2$subject!="M_pp17"&d2$subject!="M_pp28"&d2$subject!="N_pp14"&d2$word!="saloon",] # technical issues + too many errors
d3=d2a[d2a$correctness=="correct"&d2a$correctPrime=="correct",]
round(100*(nrow(d2a)-nrow(d3))/nrow(d2a)) # 10% of data

a=setdiffDF(d2a, d3)
summary(a$nativeLanguage)
# dutch english spanish 
# 118      62     153 

d3a=d3
d4=d3a[d3a$RTprev!="-",] 
d4$RTprev = as.numeric(as.character(d4$RTprev))
d4=d4[d4$RTprev>0,]
d4$lag = as.numeric(as.character(d4$lag))
d4$wavduration = as.numeric(as.character(d4$wavduration))
d4$logRTprev = log(d4$RTprev)
d4$logRT = log(d4$RT)
d4$logRTprime = log(d4$RTprime)    
d4$logwavduration = log(d4$wavduration)  
d5=d4[d4$logRT<mean(d4$logRT)+2.5*sd(d4$logRT)&d4$logRT>mean(d4$logRT)-2.5*sd(d4$logRT),] 

round(100*(nrow(d4)-nrow(d5))/nrow(d4)) #3 % of data 
a=setdiffDF(d4, d5)
summary(a$nativeLanguage)
# dutch english spanish 
# 27      20      36 

#### average RTs

summary(d5$RT)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 558.0   806.0   902.0   936.1  1024.0  1656.0
round(sd(d5$RT))
# [1] 186

#### final RT model

d5$nativeLanguage=relevel(d5$nativeLanguage, "english")
d5$groupcontrast=d5$nativeLanguage
d5$condition=relevel(d5$condition, 'speaker_mismatch') # match on intercept
contrasts(d5$groupcontrast) = rbind(c(2/3, 0), c(-1/3, 1/2), c(-1/3, -1/2)) # helmert coding, assumes equal group N
contrasts(d5$groupcontrast)
#               [,1] [,2]
# english  0.6666667  0.0
# dutch   -0.3333333  0.5
# spanish -0.3333333 -0.5

cor.test(d5$logRTprime,d5$logRTprev)
# Pearson's product-moment correlation
# data:  d5$logRTprime and d5$logRTprev
# t = 15.516, df = 2759, p-value < 2.2e-16
# alternative hypothesis: true correlation is not equal to 0
# 95 percent confidence interval:
#   0.2486094 0.3172368
# sample estimates:
#   cor 
# 0.2832857             #  not highly correlated: both may be included in the same model

# final model
d5.lmer2 =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration +  (1|word) + (1|subject), data=d5, REML=F)
d5.lmer2a = lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration +   (1|word) + (1|subject), data=d5[abs(scale(resid(d5.lmer2))) < 2.5,], REML=F)
summary(d5.lmer2a)
# Random effects:
#   Groups   Name        Variance Std.Dev.
# subject  (Intercept) 0.004852 0.06966 
# word     (Intercept) 0.002034 0.04510 
# Residual             0.017440 0.13206 
# Number of obs: 2699, groups:  subject, 110; word, 29
# 
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)                            4.206229   0.481831   8.730
# conditionspeaker_match                -0.003596   0.005197  -0.692
# groupcontrast1                        -0.067219   0.016029  -4.194
# groupcontrast2                        -0.004922   0.019330  -0.255
# logRTprev                              0.062806   0.011264   5.576
# logRTprime                             0.179542   0.014338  12.522
# logwavduration                         0.147701   0.073632   2.006
# conditionspeaker_match:groupcontrast1 -0.022363   0.010561  -2.118
# conditionspeaker_match:groupcontrast2 -0.011806   0.013247  -0.891

d5$condition_rel = relevel(d5$condition,"speaker_match")
d5.lmer2_rel =  lmer(logRT~ condition_rel * groupcontrast + logRTprev + logRTprime + logwavduration +   (1|word) + (1|subject), data=d5, REML=F)
d5.lmer2a_rel = lmer(logRT~ condition_rel * groupcontrast + logRTprev + logRTprime + logwavduration +   (1|word) + (1|subject), data=d5[abs(scale(resid(d5.lmer2))) < 2.5,], REML=F)
summary(d5.lmer2a_rel)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)                                   4.202633   0.481815   8.722
# condition_relspeaker_mismatch                 0.003596   0.005197   0.692
# groupcontrast1                               -0.089582   0.015954  -5.615
# groupcontrast2                               -0.016728   0.019263  -0.868
# logRTprev                                     0.062806   0.011264   5.576
# logRTprime                                    0.179542   0.014338  12.522
# logwavduration                                0.147701   0.073632   2.006
# condition_relspeaker_mismatch:groupcontrast1  0.022363   0.010561   2.118
# condition_relspeaker_mismatch:groupcontrast2  0.011806   0.013247   0.891

# split on contrast 1 (native vs. non-native)
# native data
dn=d5[d5$nativeLanguage=="english",]
dn.lmer2 =  lmer(logRT~ condition + logRTprev + logRTprime + logwavduration + (1|word) + (1|subject), data=dn, REML=F)
dn.lmer2a = lmer(logRT~ condition + logRTprev + logRTprime + logwavduration + (1|word) + (1|subject), data=dn[abs(scale(resid(dn.lmer2))) < 2.5,], REML=F)
summary(dn.lmer2a)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)             4.716823   0.617685   7.636
# conditionspeaker_match -0.017756   0.008335  -2.130 (in-text)
# logRTprev               0.094694   0.020231   4.681
# logRTprime              0.084623   0.025743   3.287
# logwavduration          0.128402   0.091662   1.401

# non-native data
dnn=d5[d5$nativeLanguage!="english",]
dnn.lmer2 =  lmer(logRT~ condition + logRTprev + logRTprime + logwavduration + (1|word) + (1|subject), data=dnn, REML=F)
dnn.lmer2a = lmer(logRT~ condition + logRTprev + logRTprime + logwavduration + (1|word) + (1|subject), data=dnn[abs(scale(resid(dnn.lmer2))) < 2.5,], REML=F)
summary(dnn.lmer2a)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)            4.052267   0.470073   8.621
# conditionspeaker_match 0.001667   0.006453   0.258 (in-text)
# logRTprev              0.046325   0.013484   3.436
# logRTprime             0.214697   0.017532  12.246
# logwavduration         0.154929   0.070485   2.198

# competing model with all fixed effects that do not lead to convergence errors
d5.lmer3 =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration + X2log + trial +  lag + (1|word) + (1|subject), data=d5, REML=F)
d5.lmer3a = lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration + X2log + trial +  lag + (1|word) + (1|subject), data=d5[abs(scale(resid(d5.lmer3))) < 2.5,], REML=F)
summary(d5.lmer3a)
# Random effects:
#   Groups   Name        Variance Std.Dev.
# subject  (Intercept) 0.004836 0.06954 
# word     (Intercept) 0.002015 0.04488 
# Residual             0.017387 0.13186 
# Number of obs: 2698, groups:  subject, 110; word, 29
# 
# Fixed effects: (A2-2)
#   Estimate Std. Error t value
# (Intercept)                            4.1818119  0.4815436   8.684
# conditionspeaker_match                -0.0038571  0.0051898  -0.743
# groupcontrast1                        -0.0672025  0.0160040  -4.199
# groupcontrast2                        -0.0049044  0.0192975  -0.254
# logRTprev                              0.0621925  0.0112653   5.521
# logRTprime                             0.1803406  0.0143316  12.583
# logwavduration                         0.1497578  0.0733680   2.041
# X2log                                 -0.0003812  0.0045282  -0.084
# trial                                  0.0001838  0.0001928   0.954
# lag                                   -0.0001098  0.0001714  -0.641
# conditionspeaker_match:groupcontrast1 -0.0231242  0.0105490  -2.192
# conditionspeaker_match:groupcontrast2 -0.0117743  0.0132277  -0.890       

d5.lmer3_rel =  lmer(logRT~ condition_rel * groupcontrast + logRTprev + logRTprime + logwavduration + X2log + trial +  lag + (1|word) + (1|subject), data=d5, REML=F)
d5.lmer3a_rel = lmer(logRT~ condition_rel * groupcontrast + logRTprev + logRTprime + logwavduration + X2log + trial +  lag + (1|word) + (1|subject), data=d5[abs(scale(resid(d5.lmer3))) < 2.5,], REML=F)
summary(d5.lmer3a_rel)
### same
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)                                   4.1779548  0.4815282   8.676
# condition_relspeaker_mismatch                 0.0038571  0.0051898   0.743
# groupcontrast1                               -0.0903267  0.0159325  -5.669
# groupcontrast2                               -0.0166787  0.0192310  -0.867
# logRTprev                                     0.0621925  0.0112653   5.521
# logRTprime                                    0.1803406  0.0143316  12.583
# logwavduration                                0.1497578  0.0733680   2.041
# X2log                                        -0.0003812  0.0045282  -0.084
# trial                                         0.0001838  0.0001928   0.954
# lag                                          -0.0001098  0.0001714  -0.641
# condition_relspeaker_mismatch:groupcontrast1  0.0231242  0.0105490   2.192
# condition_relspeaker_mismatch:groupcontrast2  0.0117743  0.0132277   0.890

# random slopes (test on final model)
# group by word: convergence failure
d5.lmer4a =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration +  (1|word) + (0+groupcontrast|word) +  (1|subject), data=d5, REML=F)

# speaker match by word: singular fit
d5.lmer4b =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime +  logwavduration + (1|word) + (0+condition|word) +  (1|subject), data=d5, REML=F)

# speaker match by pp: singular fit
d5.lmer4c =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime +  logwavduration + (1|word)  +  (1|subject) + (0+condition|subject), data=d5, REML=F)

## additional analysis: cognate status (on separate DU & SP RT data)
# Dutch data
cognateD=c("balloon","banana","career", "cassette", "cement", "committee", 
           "debate", "defect","defence","domain","guitar","machine","parade",
           "police","safari","salami","saloon","tobacco","tomato")
d5$cognateD=F
d5[is.element(d5$word, cognateD),]$cognateD=T
du=d5[d5$nativeLanguage=="dutch",]

du.lmer2 = lmer(logRT ~   condition * cognateD +  logRTprime + logRTprev + logwavduration + (1|word) + (1|subject), data=du, REML=F)
du.lmer2a = lmer(logRT~   condition * cognateD +  logRTprime + logRTprev + logwavduration + (1|word) + (1|subject), data=du[abs(scale(resid(du.lmer2))) < 2.5,], REML=F)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)                          3.645964   0.507802   7.180
# conditionspeaker_match               0.002994   0.011323   0.264
# cognateDTRUE                         0.003327   0.019626   0.170
# logRTprime                           0.196817   0.022649   8.690
# logRTprev                            0.059523   0.017659   3.371
# logwavduration                       0.222857   0.074263   3.001
# conditionspeaker_match:cognateDTRUE -0.010370   0.014762  -0.702


# Spanish data
cognateSP=c("career", "cassette", "cement","collapse","committee", 
            "debate","decline", "defect","defence","design","divorce","domain","guitar","machine","parade",
            "police","potato","safari","salami","saloon","surprise","tobacco","tomato")
d5$cognateSP=F
d5[is.element(d5$word, cognateSP),]$cognateSP=T
sp=d5[d5$nativeLanguage=="spanish",]
sp.lmer2 = lmer(logRT ~   condition * cognateSP +  logRTprime + logRTprev + logwavduration + (1|word) + (1|subject), data=sp, REML=F)
sp.lmer2a = lmer(logRT~   condition * cognateSP +  logRTprime + logRTprev + logwavduration + (1|word) + (1|subject), data=sp[abs(scale(resid(sp.lmer2))) < 2.5,], REML=F)
# Fixed effects: (in-text)
#   Estimate Std. Error t value
# (Intercept)                           4.34600    0.58276   7.458
# conditionspeaker_match                0.03024    0.02380   1.271
# cognateSPTRUE                        -0.01219    0.02793  -0.436
# logRTprime                            0.23248    0.02806   8.284
# logRTprev                             0.03995    0.02090   1.911
# logwavduration                        0.09748    0.08409   1.159
# conditionspeaker_match:cognateSPTRUE -0.02044    0.02752  -0.743


## additional analysis: proficiency 
# subsets of non-native listeners with similar proficiencies:
cl=d5[d5$nativeLanguage=="dutch"&d5$score_lextale<=81.25|d5$nativeLanguage=="spanish"&d5$score_lextale>51.25,]
# random sample of 29 English pp (obtained with function sample(englishdata,size=29)).
es=c("C_pp32", "C_pp26", "C_pp18", "C_pp16", "C_pp36", "C_pp8",  "C_pp6",  "C_pp29", "C_pp40", "C_pp15", "C_pp2", "C_pp1","C_pp11", "C_pp19", 
     "C_pp17","C_pp21", "C_pp35", "C_pp28", "C_pp38", "C_pp34", "C_pp22", "C_pp33", "C_pp24", "C_pp23", "C_pp5",  "C_pp25","C_pp30", "C_pp27", "C_pp20")
# select this sample from larger ds
e3 = d5[is.element(d5$subject, es),]
# combine native and non-native samples
cl2=rbind(cl,e3)

cl2$groupcontrast=cl2$nativeLanguage
contrasts(cl2$groupcontrast) = rbind(c(2/3, 0), c(-1/3, 1/2), c(-1/3, -1/2)) # helmert coding

# apply final model to this subset
cl2.lmer2 =  lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration +  (1|word) + (1|subject), data=cl2, REML=F)
cl2.lmer2a = lmer(logRT~ condition * groupcontrast + logRTprev + logRTprime + logwavduration +  (1|word) + (1|subject), data=cl2[abs(scale(resid(cl2.lmer2))) < 2.5,], REML=F)
summary(cl2.lmer2a)
# Fixed effects:
#   Estimate Std. Error t value
# (Intercept)                            4.205530   0.503340   8.355
# conditionspeaker_match                -0.004949   0.005888  -0.840
# groupcontrast1                        -0.066426   0.017331  -3.833
# groupcontrast2                        -0.004706   0.019976  -0.236
# logRTprev                              0.060774   0.012508   4.859
# logRTprime                             0.180928   0.016033  11.285
# logwavduration                         0.148360   0.076518   1.939
# conditionspeaker_match:groupcontrast1 -0.031578   0.012232  -2.582
# conditionspeaker_match:groupcontrast2 -0.005373   0.014728  -0.365
# --> similar results as in final model of full dataset

### code for barplots RT

### for plotting 
library(ggplot2)
library(gridExtra)
library(ggpubr)

d5=droplevels(d5)
levels(d5$condition)# "speaker_mismatch" "speaker_match" 
levels(d5$condition)=c("Speaker mismatch", "Speaker match")
d5$condition=relevel(d5$condition, "Speaker match")

levels(d5$nativeLanguage) #"english" "dutch"   "spanish"

levels(d5$nativeLanguage)=c("English","Dutch","Spanish")
d5a <- summarySEwithin(d5, measurevar="RT", withinvars=c("nativeLanguage","condition"), idvar="subject")
d5a
# nativeLanguage        condition   N       RT  RT_norm       sd        se       ci
# 1          Dutch    Speaker match 487 953.6858 934.6064 159.8280  7.242498 14.23047
# 2          Dutch Speaker mismatch 495 959.1374 937.4953 152.6810  6.862500 13.48329
# 3        English    Speaker match 533 875.2927 930.5579 174.0951  7.540894 14.81358
# 4        English Speaker mismatch 543 885.4659 941.4661 163.7879  7.028813 13.80705
# 5        Spanish    Speaker match 358 996.6760 939.4505 199.3598 10.536495 20.72140
# 6        Spanish Speaker mismatch 345 988.7014 932.5471 191.2097 10.294383 20.24786


pl2<- ggplot(d5a, aes(x=nativeLanguage, y=RT,fill=condition)) + 
  geom_bar(stat="identity",color="black",
           position=position_dodge(), size=0.75) +
  geom_errorbar(aes(ymin=RT-ci, ymax=RT+ci), width=.2,
                position=position_dodge(.9),size=0.75) 
#scale_y_continuous(limits=c(500, 1300),size=2) +
scale_fill_grey() 
print(pl2)
pl2 = ggpar(pl2, palette="grey",size=2,main="", ylab="Target RT (ms)", xlab="Native language", ylim=c(500,1050), legend="right", legend.title="", font.legend=c(16, "black"), 
            font.x=16, font.y=16, font.tickslab=14, ggtheme=theme_minimal())
pl2


### condition is WITHIN pp variables
# so SE from mean must also be calculated as such
# see http://www.cookbook-r.com/Graphs/Plotting_means_and_error_bars_(ggplot2)/


############# HELPER FUNCTIONS
summarySE <- function(data=NULL, measurevar, groupvars=NULL, na.rm=FALSE,
                      conf.interval=.95, .drop=TRUE) {
  library(plyr)
  
  # New version of length which can handle NA's: if na.rm==T, don't count them
  length2 <- function (x, na.rm=FALSE) {
    if (na.rm) sum(!is.na(x))
    else       length(x)
  }
  
  # This does the summary. For each group's data frame, return a vector with
  # N, mean, and sd
  datac <- ddply(data, groupvars, .drop=.drop,
                 .fun = function(xx, col) {
                   c(N    = length2(xx[[col]], na.rm=na.rm),
                     mean = mean   (xx[[col]], na.rm=na.rm),
                     sd   = sd     (xx[[col]], na.rm=na.rm)
                   )
                 },
                 measurevar
  )
  
  # Rename the "mean" column    
  datac <- rename(datac, c("mean" = measurevar))
  
  datac$se <- datac$sd / sqrt(datac$N)  # Calculate standard error of the mean
  
  # Confidence interval multiplier for standard error
  # Calculate t-statistic for confidence interval: 
  # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
  ciMult <- qt(conf.interval/2 + .5, datac$N-1)
  datac$ci <- datac$se * ciMult
  
  return(datac)
}




normDataWithin <- function(data=NULL, idvar, measurevar, betweenvars=NULL,
                           na.rm=FALSE, .drop=TRUE) {
  library(plyr)
  
  # Measure var on left, idvar + between vars on right of formula.
  data.subjMean <- ddply(data, c(idvar, betweenvars), .drop=.drop,
                         .fun = function(xx, col, na.rm) {
                           c(subjMean = mean(xx[,col], na.rm=na.rm))
                         },
                         measurevar,
                         na.rm
  )
  
  # Put the subject means with original data
  data <- merge(data, data.subjMean)
  
  # Get the normalized data in a new column
  measureNormedVar <- paste(measurevar, "_norm", sep="")
  data[,measureNormedVar] <- data[,measurevar] - data[,"subjMean"] +
    mean(data[,measurevar], na.rm=na.rm)
  
  # Remove this subject mean column
  data$subjMean <- NULL
  
  return(data)
}


summarySEwithin <- function(data=NULL, measurevar, betweenvars=NULL, withinvars=NULL,
                            idvar=NULL, na.rm=FALSE, conf.interval=.95, .drop=TRUE) {
  
  # Ensure that the betweenvars and withinvars are factors
  factorvars <- vapply(data[, c(betweenvars, withinvars), drop=FALSE],
                       FUN=is.factor, FUN.VALUE=logical(1))
  
  if (!all(factorvars)) {
    nonfactorvars <- names(factorvars)[!factorvars]
    message("Automatically converting the following non-factors to factors: ",
            paste(nonfactorvars, collapse = ", "))
    data[nonfactorvars] <- lapply(data[nonfactorvars], factor)
  }
  
  # Get the means from the un-normed data
  datac <- summarySE(data, measurevar, groupvars=c(betweenvars, withinvars),
                     na.rm=na.rm, conf.interval=conf.interval, .drop=.drop)
  
  # Drop all the unused columns (these will be calculated with normed data)
  datac$sd <- NULL
  datac$se <- NULL
  datac$ci <- NULL
  
  # Norm each subject's data
  ndata <- normDataWithin(data, idvar, measurevar, betweenvars, na.rm, .drop=.drop)
  
  # This is the name of the new column
  measurevar_n <- paste(measurevar, "_norm", sep="")
  
  # Collapse the normed data - now we can treat between and within vars the same
  ndatac <- summarySE(ndata, measurevar_n, groupvars=c(betweenvars, withinvars),
                      na.rm=na.rm, conf.interval=conf.interval, .drop=.drop)
  
  # Apply correction from Morey (2008) to the standard error and confidence interval
  #  Get the product of the number of conditions of within-S variables
  nWithinGroups    <- prod(vapply(ndatac[,withinvars, drop=FALSE], FUN=nlevels,
                                  FUN.VALUE=numeric(1)))
  correctionFactor <- sqrt( nWithinGroups / (nWithinGroups-1) )
  
  # Apply the correction factor
  ndatac$sd <- ndatac$sd * correctionFactor
  ndatac$se <- ndatac$se * correctionFactor
  ndatac$ci <- ndatac$ci * correctionFactor
  
  # Combine the un-normed means with the normed results
  merge(datac, ndatac)
}



